==> mthree-0.22.0-cp38-cp38-macosx_10_9_x86_64.whl: mthree/test/test_utils.py <==
# This code is part of Mthree.
#
# (C) Copyright IBM 2021.
#
# This code is licensed under the Apache License, Version 2.0. You may
# obtain a copy of this license in the LICENSE.txt file in the root directory
# of this source tree or at http://www.apache.org/licenses/LICENSE-2.0.
#
# Any modifications or derivative works of this code must retain this
# copyright notice, and modified files need to carry a notice indicating
# that they have been altered from the originals.
# pylint: disable=no-name-in-module

"""Test utils functions"""
import numpy as np
from qiskit import QuantumCircuit, execute
from qiskit.quantum_info import Statevector
from qiskit.test.mock import FakeAthens
import mthree


def test_gen_dist0():
    """Verify that expval of 1 circuit raw counts gives same as dist=0 solution."""
    backend = FakeAthens()
    qc = QuantumCircuit(4)
    qc.h(2)
    qc.cx(2, 1)
    qc.cx(2, 3)
    qc.cx(1, 0)
    qc.measure_all()

    raw_counts = execute(qc, backend).result().get_counts()
    mit = mthree.M3Mitigation(backend)
    mit.cals_from_system()
    mit_counts = mit.apply_correction(raw_counts, qubits=range(4),
                                      return_mitigation_overhead=True,
                                      distance=0)

    assert np.allclose(mthree.utils.expval(raw_counts), mit_counts.expval())
    assert np.allclose(mthree.utils.expval(mit_counts), mit_counts.expval())
    assert np.allclose(mthree.utils.expval(mit_counts, 'IZZI'), mit_counts.expval('IZZI'))
    assert np.allclose(mthree.utils.stddev(raw_counts), mit_counts.stddev())


def test_gen_multi_dist0():
    """Verify that expval of multi circuit raw counts gives same as dist=0 solution."""
    backend = FakeAthens()
    qc = QuantumCircuit(4)
    qc.h(2)
    qc.cx(2, 1)
    qc.cx(2, 3)
    qc.cx(1, 0)
    qc.measure_all()

    raw_counts = execute([qc]*5, backend).result().get_counts()
    mit = mthree.M3Mitigation(backend)
    mit.cals_from_system()
    mit_counts = mit.apply_correction(raw_counts, qubits=range(4),
                                      return_mitigation_overhead=True,
                                      distance=0)

    assert np.allclose(mthree.utils.expval(raw_counts), mit_counts.expval())
    assert np.allclose(mthree.utils.expval(mit_counts), mit_counts.expval())
    assert np.allclose(mthree.utils.expval(mit_counts, 'IZZI'), mit_counts.expval('IZZI'))
    dicts = [dict(rc) for rc in raw_counts]
    assert np.allclose(mthree.utils.expval(dicts), mit_counts.expval())
    assert np.allclose(mthree.utils.stddev(raw_counts), mit_counts.stddev())


def test_gen_full_dist():
    """Verify that things work for non-trivial mitigation"""
    backend = FakeAthens()
    qc = QuantumCircuit(4)
    qc.h(2)
    qc.cx(2, 1)
    qc.cx(2, 3)
    qc.cx(1, 0)
    qc.measure_all()

    raw_counts = execute(qc, backend).result().get_counts()
    mit = mthree.M3Mitigation(backend)
    mit.cals_from_system()
    mit_counts = mit.apply_correction(raw_counts, qubits=range(4),
                                      return_mitigation_overhead=True)

    assert np.allclose(mthree.utils.expval(mit_counts), mit_counts.expval())
    assert np.allclose(mthree.utils.stddev(mit_counts), mit_counts.stddev())

    probs = mit_counts.nearest_probability_distribution()
    assert np.allclose(mthree.utils.expval(probs), probs.expval())
    assert np.allclose(mthree.utils.stddev(probs), probs.stddev())


def test_gen_multi_full_dist():
    """Verify that things work for non-trivial mitigation of multi circuits"""
    backend = FakeAthens()
    qc = QuantumCircuit(4)
    qc.h(2)
    qc.cx(2, 1)
    qc.cx(2, 3)
    qc.cx(1, 0)
    qc.measure_all()

    raw_counts = execute([qc]*5, backend).result().get_counts()
    mit = mthree.M3Mitigation(backend)
    mit.cals_from_system()
    mit_counts = mit.apply_correction(raw_counts, qubits=range(4),
                                      return_mitigation_overhead=True)

    assert np.allclose(mthree.utils.expval(mit_counts), mit_counts.expval())
    assert np.allclose(mthree.utils.stddev(mit_counts), mit_counts.stddev())

    probs = mit_counts.nearest_probability_distribution()
    assert np.allclose(mthree.utils.expval(probs), probs.expval())
    assert np.allclose(mthree.utils.stddev(probs), probs.stddev())


def test_statevector_probs():
    """Verify that probabilities from statevectors work"""
    N = 4
    qc = QuantumCircuit(N)
    qc.x(range(4))
    qc.h(range(4))
    qc.ch(2, 1)
    qc.ch(1, 0)
    qc.ch(2, 3)

    st = Statevector.from_instruction(qc)
    probs = st.probabilities_dict()
    assert np.allclose(mthree.utils.expval(probs), 0.5)

==> mthree-2.6.3-cp38-cp38-macosx_10_15_x86_64.whl: mthree/test/test_utils.py <==
# This code is part of Mthree.
#
# (C) Copyright IBM 2021.
#
# This code is licensed under the Apache License, Version 2.0. You may
# obtain a copy of this license in the LICENSE.txt file in the root directory
# of this source tree or at http://www.apache.org/licenses/LICENSE-2.0.
#
# Any modifications or derivative works of this code must retain this
# copyright notice, and modified files need to carry a notice indicating
# that they have been altered from the originals.
# pylint: disable=no-name-in-module

"""Test utils functions"""
import numpy as np
from qiskit import QuantumCircuit
from qiskit.quantum_info import Statevector
from qiskit_ibm_runtime.fake_provider import FakeAthens
import mthree


def test_gen_dist0():
    """Verify that expval of 1 circuit raw counts gives same as dist=0 solution."""
    backend = FakeAthens()
    qc = QuantumCircuit(4)
    qc.h(2)
    qc.cx(2, 1)
    qc.cx(2, 3)
    qc.cx(1, 0)
    qc.measure_all()

    raw_counts = backend.run(qc).result().get_counts()
    mit = mthree.M3Mitigation(backend)
    mit.cals_from_system()
    mit_counts = mit.apply_correction(raw_counts, qubits=range(4),
                                      return_mitigation_overhead=True,
                                      distance=0)

    assert np.allclose(mthree.utils.expval(raw_counts), mit_counts.expval())
    assert np.allclose(mthree.utils.expval(mit_counts), mit_counts.expval())
    assert np.allclose(mthree.utils.expval(mit_counts, 'IZZI'), mit_counts.expval('IZZI'))
    assert np.allclose(mthree.utils.stddev(raw_counts), mit_counts.stddev())


def test_gen_multi_dist0():
    """Verify that expval of multi circuit raw counts gives same as dist=0 solution."""
    backend = FakeAthens()
    qc = QuantumCircuit(4)
    qc.h(2)
    qc.cx(2, 1)
    qc.cx(2, 3)
    qc.cx(1, 0)
    qc.measure_all()

    raw_counts = backend.run([qc]*5).result().get_counts()
    mit = mthree.M3Mitigation(backend)
    mit.cals_from_system()
    mit_counts = mit.apply_correction(raw_counts, qubits=range(4),
                                      return_mitigation_overhead=True,
                                      distance=0)

    assert np.allclose(mthree.utils.expval(raw_counts), mit_counts.expval())
    assert np.allclose(mthree.utils.expval(mit_counts), mit_counts.expval())
    assert np.allclose(mthree.utils.expval(mit_counts, 'IZZI'), mit_counts.expval('IZZI'))
    dicts = [dict(rc) for rc in raw_counts]
    assert np.allclose(mthree.utils.expval(dicts), mit_counts.expval())
    assert np.allclose(mthree.utils.stddev(raw_counts), mit_counts.stddev())


def test_gen_full_dist():
    """Verify that things work for non-trivial mitigation"""
    backend = FakeAthens()
    qc = QuantumCircuit(4)
    qc.h(2)
    qc.cx(2, 1)
    qc.cx(2, 3)
    qc.cx(1, 0)
    qc.measure_all()

    raw_counts = backend.run(qc).result().get_counts()
    mit = mthree.M3Mitigation(backend)
    mit.cals_from_system()
    mit_counts = mit.apply_correction(raw_counts, qubits=range(4),
                                      return_mitigation_overhead=True)

    assert np.allclose(mthree.utils.expval(mit_counts), mit_counts.expval())
    assert np.allclose(mthree.utils.stddev(mit_counts), mit_counts.stddev())

    probs = mit_counts.nearest_probability_distribution()
    assert np.allclose(mthree.utils.expval(probs), probs.expval())
    assert np.allclose(mthree.utils.stddev(probs), probs.stddev())


def test_gen_multi_full_dist():
    """Verify that things work for non-trivial mitigation of multi circuits"""
    backend = FakeAthens()
    qc = QuantumCircuit(4)
    qc.h(2)
    qc.cx(2, 1)
    qc.cx(2, 3)
    qc.cx(1, 0)
    qc.measure_all()

    raw_counts = backend.run([qc]*5).result().get_counts()
    mit = mthree.M3Mitigation(backend)
    mit.cals_from_system()
    mit_counts = mit.apply_correction(raw_counts, qubits=range(4),
                                      return_mitigation_overhead=True)

    assert np.allclose(mthree.utils.expval(mit_counts), mit_counts.expval())
    assert np.allclose(mthree.utils.stddev(mit_counts), mit_counts.stddev())

    probs = mit_counts.nearest_probability_distribution()
    assert np.allclose(mthree.utils.expval(probs), probs.expval())
    assert np.allclose(mthree.utils.stddev(probs), probs.stddev())


def test_statevector_probs():
    """Verify that probabilities from statevectors work"""
    N = 4
    qc = QuantumCircuit(N)
    qc.x(range(4))
    qc.h(range(4))
    qc.ch(2, 1)
    qc.ch(1, 0)
    qc.ch(2, 3)

    st = Statevector.from_instruction(qc)
    probs = st.probabilities_dict()
    assert np.allclose(mthree.utils.expval(probs), 0.5)
